import requests
import json
from pyoauthbridge.server import Server
from threading import Thread
from pyoauthbridge.wsclient import socket_connect, get_compact_marketdata, get_detailed_marketdata, get_snapquotedata, send_message, get_ws_connection_status, unsubscribe_update
import sys
import time

cli = sys.modules['flask.cli']
cli.show_server_banner = lambda *x: None
class Connect:
    def __init__(self, client_id, client_secret, redirect_url, base_url):
        self.headers = {'Content-type': 'application/json'}
        self.access_token = ""
        self.login_id = ""
        self.base_url=base_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_url = redirect_url
        url = ""
        if "https" in base_url:
            url = base_url.replace("https", "wss")
        else:
            url = base_url.replace("http", "ws")
        self.websocket_url = url

    def get_access_token(self):
        base_url = self.base_url
        client_id = self.client_id
        client_secret = self.client_secret
        redirect_url = self.redirect_url
        server = Server(client_id, client_secret, redirect_url, base_url)
        app = server.create_app()
        app.env = 'development'
        print('Open this url in browser:', 'http://127.0.0.1:65010/getcode', end='\n\n')
        app.run(host='127.0.0.1', debug=False, port=65010)
        access_token = server.fetch_access_token()
        self.access_token = access_token
        return server.fetch_access_token()

    def print_access_token(self):
        return self.access_token

    def set_access_token(self, access_token):
        self.access_token = access_token

    def get_request(self, url, params):
        headers = self.headers
        headers['Authorization'] = f'Bearer {self.access_token}'
        res = requests.get(f'{self.base_url}{url}' , params=params, headers=headers)
        return res.json()

    def post_request(self, url, data):
        headers = self.headers
        headers['Authorization'] = f'Bearer {self.access_token}'
        res = requests.post(f'{self.base_url}{url}', headers=headers, data=json.dumps(data))
        print(res)
        return res.json()

    def put_request(self, url, data):
        headers = self.headers
        headers['Authorization'] = f'Bearer {self.access_token}'
        res = requests.put(f'{self.base_url}{url}', headers=headers, data=json.dumps(data))
        print(res)
        return res.json()

    def delete_request(self, url, params):
        headers = self.headers
        headers['Authorization'] = f'Bearer {self.access_token}'
        res = requests.delete(f'{self.base_url}{url}' , params=params, headers=headers)
        return res.json()

    def fetch_profile(self, payload):
        params = {'client_id': payload['client_id']}
        res = self.get_request("/api/v1/user/profile", params)
        return res

    def place_order(self, payload):
        data = {
            "exchange": payload['exchange'],
            "instrument_token": payload['instrument_token'],
            "client_id": payload['client_id'],
            "order_type": payload['order_type'],
            "amo": payload['amo'],
            "price": payload['price'],
            "quantity": payload['quantity'],
            "disclosed_quantity": payload['disclosed_quantity'],
            "validity": payload['validity'],
            "product": payload['product'],
            "order_side": payload['order_side'],
            "device": "api",
            "user_order_id": payload['user_order_id'],
            "trigger_price": payload['trigger_price'],
            "execution_type": payload['execution_type']
        }
        res = self.post_request(f'/api/v1/orders', data)
        return res

    def modify_order(self, payload):
        data = {
            "exchange": payload['exchange'],
            "instrument_token": payload['instrument_token'],
            "client_id": payload['client_id'],
            "order_type": payload['order_type'],
            "price": payload['price'],
            "quantity": payload['quantity'],
            "disclosed_quantity": payload['disclosed_quantity'],
            "validity": payload['validity'],
            "product": payload['product'],
            "oms_order_id": payload['oms_order_id'],
            "trigger_price": payload['trigger_price'],
            "execution_type": payload['execution_type']
        }
        res = self.put_request("/api/v1/orders", data)
        return res

    def cancel_order(self, payload):
        params = {
            'client_id': payload['client_id'],
            'execution_type': payload['execution_type']
        }
        oms_order_id = payload['oms_order_id']
        res = self.delete_request(f'/api/v1/orders/{oms_order_id}', params)
        return res

    def fetch_scripinfo(self, payload):
        params = {
            'info': 'scrip',
            'token': payload['token']
        }
        exchange = payload["exchange"]
        res = self.get_request(f'/api/v1/contract/{exchange}', params)
        return res

    def search_scrip(self, payload):
        params = {
            'key': payload['key']
        }
        res = self.get_request(f'/api/v1/search', params)
        return res

    def fetch_pending_orders(self, payload):
        params = {
            'type': 'pending',
            'client_id': payload['client_id']
        }
        res = self.get_request(f'/api/v1/orders', params)
        return res

    def fetch_completed_orders(self, payload):
        params = {
            'type': 'completed',
            'client_id': payload['client_id']
        }
        res = self.get_request(f'/api/v1/orders', params)
        return res

    def fetch_trades(self, payload):
        params = {
            'client_id': payload['client_id']
        }
        res = self.get_request(f'/api/v1/trades', params)
        return res

    def fetch_order_history(self, payload):
        params = {
            'client_id': payload['client_id']
        }
        res = self.get_request(f'/api/v1/order/{payload.oms_order_id}/history', params)
        return res

    def fetch_live_positions(self, payload):
        params = {
            'client_id': payload['client_id'],
            'type': 'live'
        }
        res = self.get_request(f'/api/v1/positions', params)
        return res

    def fetch_netwise_positions(self, payload):
        params = {
            'client_id': payload['client_id'],
            'type': 'historical'
        }
        res = self.get_request(f'/api/v1/positions', params)
        return res

    def fetch_holdings(self, payload):
        params = {
            'client_id': payload['client_id']
        }
        res = self.get_request(f'/api/v1/holdings', params)
        return res

    def fetch_funds_v2(self, payload):
        params = {
            'client_id': payload['client_id'],
            'type': 'all'
        }
        res = self.get_request(f'/api/v2/funds/view', params)
        return res

    def fetch_funds_v1(self, payload):
        params = {
            'client_id': payload['client_id'],
            'type': 'all'
        }
        res = self.get_request(f'/api/v1/funds/view', params)
        return res

    def create_alert(self, payload):
        data = {
            'exchange': payload['exchange'],
            'instrument_token': payload['instrument_token'],
            'wait_time': payload['wait_time'],
            'condition': payload['condition'],
            'user_set_values': payload['user_set_values'],
            'frequency': payload['frequency'],
            'expiry': payload['expiry'],
            'state_after_expiry': payload['state_after_expiry'],
            'user_message': payload['user_message']
        }
        res = self.post_request(f'/api/v1/alerts', data)
        return res

    def fetch_alerts(self):
        params = {}
        res = self.get_request(f'/api/v1/alerts', params)
        return res

    def update_alert(self, payload):
        data = {
            'exchange': payload['exchange'],
            'instrument_token': payload['instrument_token'],
            'wait_time': payload['wait_time'],
            'condition': payload['condition'],
            'user_set_values': payload['user_set_values'],
            'frequency': payload['frequency'],
            'expiry': payload['expiry'],
            'state_after_expiry': payload['state_after_expiry'],
            'user_message': payload['user_message']
        }
        res = self.put_request(f'/api/v1/alerts', data)
        return res

    def run_socket(self):
        client_id = self.client_id
        access_token = self.access_token
        websocket_url = self.websocket_url
        th_websocket = Thread(target=socket_connect, args=(client_id, access_token, websocket_url,))
        th_websocket.start()
        counter = 0
        while True:
            status = get_ws_connection_status()
            if status == True:
                return True
            time.sleep(1)
            counter = counter + 1
            if counter > 5:
                return False
        # socket_connect(client_id, access_token, websocket_url)

    def subscribe_detailed_marketdata(self, detailedmarketdata_payload):
        th_detailed_marketdata = Thread(target=send_message, args=('DetailedMarketDataMessage', detailedmarketdata_payload))
        th_detailed_marketdata.start()

    def read_detailed_marketdata(self):
        data = get_detailed_marketdata()
        return data

    def unsubscribe_detailed_marketdata(self, detailedmarketdata_payload):
        th_detailed_marketdata = Thread(target=unsubscribe_update, args=('DetailedMarketDataMessage', detailedmarketdata_payload))
        th_detailed_marketdata.start()

    def subscribe_compact_marketdata(self, compactmarketdata_payload):
        th_compact_marketdata = Thread(target=send_message, args=('CompactMarketDataMessage', compactmarketdata_payload))
        th_compact_marketdata.start()

    def unsubscribe_compact_marketdata(self, compactmarketdata_payload):
        th_compact_marketdata = Thread(target=unsubscribe_update, args=('CompactMarketDataMessage', compactmarketdata_payload))
        th_compact_marketdata.start()

    def read_compact_marketdata(self):
        data = get_compact_marketdata()
        return data

    def subscribe_snapquote_data(self, snapquotedata_payload):
        th_snapquote = Thread(target=send_message, args=('SnapquoteDataMessage', snapquotedata_payload))
        th_snapquote.start()

    def unsubscribe_snapquote_data(self, snapquotedata_payload):
        th_snapquote = Thread(target=unsubscribe_update, args=('SnapquoteDataMessage', snapquotedata_payload))
        th_snapquote.start()

    def read_snapquote_data(self):
        data = get_snapquotedata()
        return data


#------------------------------------------------
#For testing
#------------------------------------------------
#
#
# if __name__ == "__main__":
#     client_id = "*******-TEST"
#     client_secret = "****************BLTBYJn10gYLYIWJqpOcn9zEYI96SJRzZ1"
#     redirect_url = "http://127.0.0.1:*****"
#     base_url = "https://***********.in"
#     conn = Connect(client_id, client_secret, redirect_url, base_url)
#     access_token = conn.get_access_token()
#
#     ##Run api
#     # payload={
#     #     "client_id": "***********"
#     # }
#     # res = conn.fetch_profile(payload)
#     # print(res)
#
#     ##Run Socket
#
#     conn.run_socket("marketdata", {
#         'exchangeCode': 1,
#         'instrumentToken': 3045
#     })




import os
import threading
import webbrowser
from requests_oauthlib import OAuth2Session
from flask import Flask, request, redirect
os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'

class Server:
    # to make oauth2 work with http;
    global_access_token = ""
    def __init__(self, client_id, client_secret, redirect_url, base_url):
        self.client_id = client_id
        self.web_url = base_url
        self.client_secret = client_secret
        self.redirect_uri = redirect_url
        self.authorization_base_url = f'{self.web_url}/oauth2/auth'
        self.token_url = f'{self.web_url}/oauth2/token'
        self.scope = 'orders holdings'
        self.app = Flask(__name__)
        self.access_token = ""

    def create_app(self):
        app = self.app
        web_url = self.web_url
        client_id = self.client_id
        redirect_uri = self.redirect_uri
        scope = self.scope
        authorization_base_url = self.authorization_base_url
        token_url = self.token_url
        client_secret = self.client_secret

        @app.route('/getcode')
        def get_authorization_url():
            oauth = OAuth2Session(client_id, redirect_uri=redirect_uri, scope=scope)
            authorization_url, _state = oauth.authorization_url(authorization_base_url, access_type="authorization_code")
            print('authorization_url')
            print(authorization_url)
            return redirect(authorization_url)

        @app.route('/')
        def callback():
            print("Inside callback function")
            oauth = OAuth2Session(client_id, redirect_uri=redirect_uri, scope=scope)
            print(self.token_url)
            token = oauth.fetch_token(token_url, authorization_response=request.url, client_secret=client_secret)

            self.access_token = token['access_token']

            func = request.environ.get('werkzeug.server.shutdown')
            if func:
                print('stoping server')
                func()

            return 'see terminal for logs'

        return app

    def fetch_access_token(self):
        return self.access_token


from connect import Connect
import webbrowser
from threading import Thread 
import time

def get_request(self, url, params):
  headers = self.headers
  headers['Authorization'] = f'Bearer {self.access_token}'
  res = requests.get(f'{self.base_url}{url}' , params=params, headers=headers)
  return res.json()

def post_request(self, url, data):
  headers = self.headers
  headers['Authorization'] = f'Bearer {self.access_token}'
  res = requests.post(f'{self.base_url}{url}', headers=headers, data=json.dumps(data))
  print(res.json())
  return res.json()

if __name__ == '__main__':
  client_id = ""
  client_secret = ""
  redirect_url = "http://127.0.0.1:65010"
  
  websocket_url = ""
  base_url = ""
  conn = Connect(client_id, client_secret, redirect_url, base_url)
  webbrowser.open("http://127.0.0.1:65010/getcode")
  access_token = conn.get_access_token()

  print(access_token)

  marketdata_payload = {'exchangeCode': 1, 'instrumentToken': 3045}
  snapquotedata_payload = {'exchangeCode': 1, 'instrumentToken': 3045}

  ws_status = conn.run_socket()

  print("websocket connected ...")
  print(ws_status)
  conn.subscribe_detailed_marketdata(marketdata_payload,)
  conn.subscribe_snapquote_data(snapquotedata_payload,)

  print("channels subscribed ....")

  i = 0
  while True:
    time.sleep(1)
    detailed_market_data = conn.read_detailed_marketdata()
    print(detailed_market_data)
    time.sleep(1)
    snapquote_data = conn.read_snapquote_data()
    print(detailed_market_data)
    i = i + 1
    print("==================================")
    if i > 5:
      print("unsubscribe marketdata")
      conn.unsubscribe_detailed_marketdata(marketdata_payload)
      print("unsubscribe snapquote")
      conn.unsubscribe_detailed_marketdata(snapquotedata_payload)
    


import ctypes, struct
from struct import pack_into

def decodeDetailedMarketData(packet_buffer):
  return {
    "mode" : struct.unpack('>b', packet_buffer[0:1])[0],
    "exchange_code" : struct.unpack('>b', packet_buffer[1:2])[0],
    "instrument_token" : struct.unpack('>I', packet_buffer[2:6])[0],
    "last_traded_price" : struct.unpack('>I', packet_buffer[6:10])[0],
    "last_traded_time" : struct.unpack('>I', packet_buffer[10:14])[0],
    "last_traded_quantity" : struct.unpack('>I', packet_buffer[14:18])[0],
    "trade_volume" : struct.unpack('>I', packet_buffer[18:22])[0],
    "best_bid_price" : struct.unpack('>I', packet_buffer[22:26])[0],
    "best_bid_quantity" : struct.unpack('>I', packet_buffer[26:30])[0],
    "best_ask_price" : struct.unpack('>I', packet_buffer[30:34])[0],
    "best_ask_quantity" : struct.unpack('>I', packet_buffer[34:38])[0],
    "total_buy_quantity" : struct.unpack('>Q', packet_buffer[38:46])[0],
    "total_sell_quantity" : struct.unpack('>Q', packet_buffer[46:54])[0],
    "average_trade_price" : struct.unpack('>I', packet_buffer[54:58])[0],
    "exchange_timestamp" : struct.unpack('>I', packet_buffer[58:62])[0],
    "open_price" : struct.unpack('>I', packet_buffer[62:66])[0],
    "high_price" : struct.unpack('>I', packet_buffer[66:70])[0],
    "low_price" : struct.unpack('>I', packet_buffer[70:74])[0],
    "close_price" : struct.unpack('>I', packet_buffer[74:78])[0],
    "yearly_high_price" : struct.unpack('>I', packet_buffer[78:82])[0],
    "yearly_low_price" : struct.unpack('>I', packet_buffer[82:86])[0],
    "lowDPR": struct.unpack('>I', packet_buffer[86:90])[0],
    "highDPR": struct.unpack('>I', packet_buffer[90:94])[0],
    "currentOpenInterest": struct.unpack('>I', packet_buffer[94:98])[0],
    "initialOpenInterest": struct.unpack('>I', packet_buffer[98:102])[0],
  }

def decodeCompactMarketData(packet_buffer):
  return {
    "mode" : struct.unpack('>b', packet_buffer[0:1])[0],
    "exchange_code" : struct.unpack('>b', packet_buffer[1:2])[0],
    "instrument_token" : struct.unpack('>I', packet_buffer[2:6])[0],
    "last_traded_price" : struct.unpack('>I', packet_buffer[6:10])[0],
    "change": struct.unpack('>I', packet_buffer[10:14])[0],
    "last_traded_time": struct.unpack('>I', packet_buffer[14:18])[0],
    "lowDPR": struct.unpack('>I', packet_buffer[18:22])[0],
    "highDPR": struct.unpack('>I', packet_buffer[22:26])[0],
    "currentOpenInterest": struct.unpack('>I', packet_buffer[26:30])[0],
    "initialOpenInterest": struct.unpack('>I', packet_buffer[30:34])[0],
    "bidPrice": struct.unpack('>I', packet_buffer[34:38])[0],
    "askPrice": struct.unpack('>I', packet_buffer[38:42])[0],
  }

def decodeSnapquoteData(packet_buffer):
  return {
    "mode" : struct.unpack('>b', packet_buffer[0:1])[0],
    "exchange_code" : struct.unpack('>b', packet_buffer[1:2])[0],
    "instrument_token" : struct.unpack('>I', packet_buffer[2:6])[0],
    "buyers": [
      struct.unpack('>I', packet_buffer[6:10])[0],
      struct.unpack('>I', packet_buffer[10:14])[0],
      struct.unpack('>I', packet_buffer[14:18])[0],
      struct.unpack('>I', packet_buffer[18:22])[0],
      struct.unpack('>I', packet_buffer[22:26])[0]
    ],
    "bidPrices": [
      struct.unpack('>I', packet_buffer[26:30])[0],
      struct.unpack('>I', packet_buffer[30:34])[0],
      struct.unpack('>I', packet_buffer[34:38])[0],
      struct.unpack('>I', packet_buffer[38:42])[0],
      struct.unpack('>I', packet_buffer[42:46])[0]
    ],
    "bidQtys": [
      struct.unpack('>I', packet_buffer[46:50])[0],
      struct.unpack('>I', packet_buffer[50:54])[0],
      struct.unpack('>I', packet_buffer[54:58])[0],
      struct.unpack('>I', packet_buffer[58:62])[0],
      struct.unpack('>I', packet_buffer[62:66])[0]
    ],
    "sellers": [
      struct.unpack('>I', packet_buffer[66:70])[0],
      struct.unpack('>I', packet_buffer[70:74])[0],
      struct.unpack('>I', packet_buffer[74:78])[0],
      struct.unpack('>I', packet_buffer[78:82])[0],
      struct.unpack('>I', packet_buffer[82:86])[0]
    ],
    "askPrices": [
      struct.unpack('>I', packet_buffer[86:90])[0],
      struct.unpack('>I', packet_buffer[90:94])[0],
      struct.unpack('>I', packet_buffer[94:98])[0],
      struct.unpack('>I', packet_buffer[98:102])[0],
      struct.unpack('>I', packet_buffer[102:106])[0]
    ],
    "askQtys": [
      struct.unpack('>I', packet_buffer[106:110])[0],
      struct.unpack('>I', packet_buffer[110:114])[0],
      struct.unpack('>I', packet_buffer[114:118])[0],
      struct.unpack('>I', packet_buffer[118:122])[0],
      struct.unpack('>I', packet_buffer[122:126])[0]
    ],
    "averageTradePrice": struct.unpack('>I', packet_buffer[126:130])[0],
    "open": struct.unpack('>I', packet_buffer[130:134])[0],
    "high": struct.unpack('>I', packet_buffer[134:138])[0],
    "low" : struct.unpack('>I', packet_buffer[138:142])[0],
    "close" : struct.unpack('>I', packet_buffer[142:146])[0],
    "totalBuyQty" : struct.unpack('>Q', packet_buffer[146:154])[0],
    "totalSellQty": struct.unpack('>Q', packet_buffer[154:162])[0],
    "volume" : struct.unpack('>I', packet_buffer[162:166])[0],
  }


import websocket
import threading
import json
import time
from struct import pack_into
import ctypes, struct
from pyoauthbridge.packetDecoder import decodeDetailedMarketData, decodeCompactMarketData, decodeSnapquoteData

login_id = ""
access_token = ""
payload = {}
websock = {}
ws_connected = False
compact_marketdata_response = {}
detailed_marketdata_response = {}
snapquote_marketdata_response = {}

def get_detailed_marketdata():
    return detailed_marketdata_response

def get_compact_marketdata():
    return compact_marketdata_response

def get_snapquotedata():
    return snapquote_marketdata_response

def get_ws_connection_status():
    return ws_connected

def heartbeat_thread(clientSocket):
    while clientSocket:
        send_data = '{"a": "h", "v": [], "m": ""}'
        try:
            clientSocket.send(send_data)
        except Exception as e:
            print(e)
            print("HEARTBEAT [ERROR]: [BLITZ_HYDRA_STREAM] Connection closed.")
            break
        print("Sent Heart-Beat to Exchange")
        time.sleep(8)


def on_message(ws, message):
    mode = struct.unpack('>b', message[0:1])[0]
    if mode == 1:
        res = decodeDetailedMarketData(message)
        # print("detailed market data")
        # print(res)
        global detailed_marketdata_response
        detailed_marketdata_response = res
    elif mode == 2:
        res = decodeCompactMarketData(message)
        global compact_marketdata_response
        compact_marketdata_response = res
    elif mode == 4:
        res = decodeSnapquoteData(message)
        # print("snap quote data")
        # print(res)
        global snapquote_marketdata_response
        snapquote_marketdata_response = res

def on_error(ws, error):
    print(error)
    global ws_connected
    ws_connected = False


def on_close(ws):
    print("### closed ###")
    global ws_connected
    ws_connected = False


def on_open(ws):

    hbThread = threading.Thread(target=heartbeat_thread, args=(ws,))
    hbThread.start()
    global ws_connected
    ws_connected = True

    # hbThread1 = threading.Thread(target=send_message, args=(ws,))
    # hbThread1.start()

def unsubscribe_update(message_type, payload):
    global websock
    clientSocket = websock
    if message_type == "DetailedMarketDataMessage":
        sub_packet = {
            "a": "unsubscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "marketdata"
        }
        clientSocket.send(json.dumps(sub_packet))
        global detailed_marketdata_response
        detailed_marketdata_response = {}
    elif message_type == "CompactMarketDataMessage":
        sub_packet = {
            "a": "unsubscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "compact_marketdata"
        }
        clientSocket.send(json.dumps(sub_packet))
        global compact_marketdata_response
        compact_marketdata_response = {}
    elif message_type == "SnapquoteDataMessage":
        sub_packet = {
            "a": "unsubscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "full_snapquote"
        }
        clientSocket.send(json.dumps(sub_packet))
        global snapquote_marketdata_response
        snapquote_marketdata_response = {}

def send_message(message_type, payload):
    # print(message_type)
    global websock
    clientSocket = websock
    print(clientSocket)
    if message_type == "DetailedMarketDataMessage":
        sub_packet = {
            "a": "subscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "marketdata"
        }
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "CompactMarketDataMessage":
        sub_packet = {
            "a": "subscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "compact_marketdata"
        }
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "SnapquoteDataMessage":
        sub_packet = {
            "a": "subscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "full_snapquote"
        }
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "TbtSnapquoteDataMessage":
        sub_packet = {
            "a": "subscribe",
            "v": [[payload['exchangeCode'], payload['instrumentToken']]],
            "m": "tbt_full_snapquote"
        }
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "OrderUpdate":
        sub_packet = {
            "a": "subscribe",
            "v": [payload['client_id'], "web"],
            "m": "updates"
        }
        print(sub_packet)
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "TradeUpdate": 
        sub_packet = {
            "a": "subscribe",
            "v": [[payload['client_id'], "web"]],
            "m": "updates"
        }
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "ExchangeMessage": 
        sub_packet = {
            "a": "subscribe",
            "v": [payload['client_id']],
            "m": "exchange_messages"
        }
        clientSocket.send(json.dumps(sub_packet))
    elif message_type == "PositionUpdate":
        sub_packet = {
            "a": "subscribe",
            "v": [[payload['client_id'], "web"]],
            "m": "position_updates"
        }
        clientSocket.send(json.dumps(sub_packet))

def connect(base_url):
    websocket.enableTrace(False)
    ws = websocket.WebSocketApp(f"{base_url}",
                                on_message=on_message,
                                on_error=on_error,
                                on_close=on_close)
    ws.on_open = on_open
    return ws

def webs_start(ws):
    ws.run_forever()

def socket_connect(client_id, token, websocket_url):

    global websock
    websock = connect(f'{websocket_url}/ws/v1/feeds?login_id={client_id}&access_token={token}')
    # print(message_type)
    hbThread2 = threading.Thread(target=webs_start, args=(websock,))
    hbThread2.run()
    hbThread2.isAlive()



import setuptools
setuptools.setup(
   name='pyoauthbridge',
   version='1.0.12',
   description='A useful module',
   long_description_content_type='text/x-rst',
   long_description='A project by tradelab',
   author='Biswajit',
   author_email='biswajit@tradelab.com',
   packages=["pyoauthbridge"]
)